from OTils.size_utils import convert_size


class MemoryTracker:
    def __init__(self):
        import psutil
        import os
        self.proc = psutil.Process(os.getpid())
        self._start = None
        self.start()

    def memory(self):
        return self.proc.memory_info().rss

    def readable(self):
        return convert_size(self.memory())

    def start(self):
        self._start = self.memory()

    @property
    def difference(self):
        return self.memory() - self._start

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        pass

    def __repr__(self):
        return f"MemoryTracker(Total: {self.readable()} Increment: {convert_size(self.difference)})"


def try_memory_tracker() -> None:
    """
    Some example code using the class MemoryTracker
    """
    import time
    experiment_duration = 2  # seconds

    # Start memory tracker out of a Context Manager
    tracker = MemoryTracker()

    # Do something for a couple of seconds
    start_time = time.perf_counter()

    # To have something to observe, lets initialize a list and fill it up in each iteration
    to_fill = []
    # Start a second tracker with a Context Manager
    while time.perf_counter() - start_time < experiment_duration:
        # Add some stuff to the list
        to_fill.append([x for x in range(100000)])
        # Print status:
        print(tracker)

        # Sleep
        time.sleep(0.5)
    pass


if __name__ == "__main__":
    try_memory_tracker()
