from typing import List


def suggester(options: List[str], prior: str):
    """
    The purpose of this function is to suggest possible matches when there's no exact match.
    Args:
        options: list of options
        prior:

    Returns:

    """
    from difflib import SequenceMatcher

    def similar(a, b):
        return SequenceMatcher(None, a, b).ratio()

    scores = {o: similar(o, prior) for o in options}

    highest_score = 0
    most_probable_answer = None
    for name, score in scores.items():
        if score > highest_score:
            most_probable_answer = name
            highest_score = score
    # In case the score is really low just return None
    if highest_score < .2:
        return None

    # Otherwise return the most probable answer
    return most_probable_answer


def try_suggester():
    """
    Sample function using suggester.
    """
    # List of possible commands inspired by autosubmit
    possible_commands = ["run", "create", "recovery", "monitor", "delete"]

    while True:
        command = input("Insert command ->$ ").strip()
        print(command)
        if command == "exit":
            print("exiting...")
            exit(0)
        elif command in possible_commands:
            print(f"Executing {command}")
        else:
            suggestion = suggester(possible_commands, command)
            if suggestion is not None:
                print(f"Command '{command}' not valid, did you mean '{suggestion}'?")
            else:
                print(f"What is {command}")


if __name__ == "__main__":
    try_suggester()
